"Config urls module"
from django.conf.urls.defaults import patterns, url
from django_restapi.responder import JsonDictResponder
from django_restapi.responder import YamlFileResponder

from django import forms

from django.shortcuts import render_to_response
from CnmResource import CnmResource
from configs.models import ServerConfig
from bombardier_core.static_data import OK, FAIL
from bombardier_core.static_data import LOCAL_TYPE, BDR_CLIENT_TYPE
import os, glob
from bombardier_server.cnm.MachineStatus import MachineStatus
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden
import yaml
from bombardier_server.cnm.Exceptions import MachineConfigurationException
from bombardier_server.cnm.Exceptions import MachineStatusException

bad_characters = ['/', '%', '\\', ' ', '$', '(', ')', '!', '#', '@',
                  '&', '*', '[', ';', '?', '|', '<', '>']

def sanitize(input_data):
    output = []
    for character in input_data:
        if character not in bad_characters:
            output.append(character)
    return ''.join(output)

class ConfigEntry(CnmResource):
    "ConfigEntry base class"
    @login_required
    def read(self, request, config_type, config_name):
        "Default read method for ConfigEntry"
        dispatcher = self.get_dispatcher()
        config_data = dispatcher.read_config_data(config_type, config_name)
        responder = JsonDictResponder(config_data)
        return responder.element(request)

    @login_required
    def create(self, request, config_type, config_name):
        "Default POST method for ConfigEntry"
        yaml_string = request.POST["yaml"]
        dispatcher = self.get_dispatcher()
        output = dispatcher.make_config_data(config_type, sanitize(config_name), yaml_string)
        responder = JsonDictResponder(output)
        return responder.element(request)

class MachineEntry(ConfigEntry):
    "Machine config entry"
    @login_required
    def read(self, request, machine_name):
        "Call superclass read method with config type"
        return super(MachineEntry, self).read(request, "machine", machine_name)

    @login_required
    def create(self, request, machine_name):
        "Call superclass read method with config type"
        return super(MachineEntry, self).create(request,
                                                "machine", machine_name)

class PackageEntry(ConfigEntry):
    "Package config entry"
    @login_required
    def read(self, request, package_name):
        "Call superclass read method with config type"
        return super(PackageEntry, self).read(request, "package", package_name)

    @login_required
    def create(self, request, package_name):
        "Call superclass read method with config type"
        return super(PackageEntry, self).create(request,
                                                "package", package_name)

class BomEntry(ConfigEntry):
    "Bom config entry"
    @login_required
    def read(self, request, bom_name):
        "Call superclass read method with config type"
        return super(BomEntry, self).read(request, "bom", bom_name)

    @login_required
    def create(self, request, bom_name):
        "Call superclass read method with config type"
        return super(BomEntry, self).create(request, "bom", bom_name)

class IncludeEntry(ConfigEntry):
    "Include config entry"
    @login_required
    def read(self, request, include_name):
        "Call superclass read method with config type"
        return super(IncludeEntry, self).read(request, "include", include_name)

    @login_required
    def create(self, request, include_name):
        "Call superclass read method with config type"
        return super(IncludeEntry, self).create(request,
                                                "include", include_name)
class SummaryEntry(CnmResource):
    "Config entry for digested summary information"
    @login_required
    def read(self, request, machine_name):
        "Digest useful information from the status of the server"
        output = {"command_status": OK}
        try:
            dispatcher = self.get_dispatcher()
            output = dispatcher.get_machine_status(machine_name)
        except Exception, x:
            output.update(self.dump_exception(request, x))
        responder = JsonDictResponder(output)
        return responder.element(request)

class StatusEntry(ConfigEntry):
    "Status config entry"
    @login_required
    def read(self, request, machine_name):
        "Call superclass read method with config type"
        return super(StatusEntry, self).read(request, "status", machine_name)

class DistEntry(ConfigEntry):
    "Config entry for dist type (used for python distutils packages)."
    @login_required
    def read(self, request, dist_name):
        "Create dictionary from Dist object and return as json"
        dispatcher = self.get_dispatcher()
        output = dispatcher.get_dist_name_info(dist_name)
        responder = JsonDictResponder(output)
        return responder.element(request)

class MergedEntry(CnmResource):
    """Merged Entry is a merged configuration based on machine and including
       based on the client configuration"""

    @login_required
    def read(self, request, machine_name):
        "Create machine config object, merge and return data as json"
        try:
            username = request.user.username
            dispatcher = self.get_dispatcher()
            machine_config = dispatcher.get_machine_config(username, machine_name, BDR_CLIENT_TYPE)
            machine_config.merge()
            responder = JsonDictResponder(machine_config.data)
        except Exception, x:
            output = self.dump_exception(request, x)
            responder = JsonDictResponder(output)
        return responder.element(request)

#==================================

class ConfigCollection(CnmResource):
    "Base config collection"
    @login_required
    def read(self, request, config_type, config_name):
        "Default read method for ConfigCollection"
        dispatcher = self.get_dispatcher()
        output = dispatcher.get_config_info(config_type, sanitize(config_name))
        responder = JsonDictResponder(output)
        return responder.element(request)

class StatusCollection(ConfigCollection):
    "Status config collection"
    @login_required
    def read(self, request, machine_name):
        "Call superclass read method with collection type"
        sup = super(StatusCollection, self)
        return sup.read(request, "status", machine_name)

class MergedCollection(ConfigCollection):
    "Merged config collection"
    @login_required
    def read(self, request, machine_name):
        "Call superclass read method with collection type"
        sup = super(MergedCollection, self)
        return sup.read(request, "machine", machine_name)

class MachineCollection(ConfigCollection):
    "Machine config collection"
    @login_required
    def read(self, request, machine_name):
        "Call superclass read method with collection type"
        sup = super(MachineCollection, self)
        return sup.read(request, "machine", machine_name)

class IncludeCollection(ConfigCollection):
    "Include config collection"
    @login_required
    def read(self, request, include_name):
        "Call superclass read method with collection type"
        sup = super(IncludeCollection, self)
        return sup.read(request, "include", include_name)

class BomCollection(ConfigCollection):
    "Bom config collection"
    @login_required
    def read(self, request, bom_name):
        "Call superclass read method with collection type"
        sup = super(BomCollection, self)
        return sup.read(request, "bom", bom_name)

class PackageCollection(ConfigCollection):
    "Package config collection"
    @login_required
    def read(self, request, package_name):
        "Call superclass read method with collection type"
        sup = super(PackageCollection, self)
        return sup.read(request, "package", package_name)

class DistCollection(ConfigCollection):
    "Dist config collection"
    @login_required
    def read(self, request, dist_name):
        "Expose custom fields for Dist type collection and return as json"
        dispatcher = self.get_dispatcher()
        output = dispatcher.get_dist_info(dist_name)
        responder = JsonDictResponder(output)
        return responder.element(request)
        #return responder.list(request, objects)

#==================================

class ServerConfigCollection(CnmResource):
    "Server config collection"
    @login_required
    def read(self, request):
        "Return json representation of server configuration"
        output = {}

        server_config = {}
        server_config_objects = ServerConfig.objects.all()
        for server_co in server_config_objects:
            server_config[server_co.name] = server_co.value
        output["server configuration"] = server_config
        responder = JsonDictResponder(output)
        return responder.element(request)

    @login_required
    def create(self, request):
        "Create or update configuration items on the server config collection."
        if not request.user.is_superuser:
            return HttpResponseForbidden()
        query_dict = request.POST

        if "form-0-name" in query_dict:
            self._form_update(query_dict)
        else:
            self._standard_update(query_dict)

        server_config_objects = ServerConfig.objects.all()
        output = {}
        for server_co in server_config_objects:
            output[server_co.name] = server_co.value
        responder = JsonDictResponder(output)
        return responder.element(request)

def config_setting_form(request):
    "Form for setting configuration items. Mostly a debugging tool."
    from django.forms.models import modelformset_factory
    ServerConfigFormSet = modelformset_factory(ServerConfig)
    server_config_objects = ServerConfig.objects.all()
    formset = ServerConfigFormSet(queryset=server_config_objects)

    return render_to_response('config_setting_form.html', {
        'formset': formset,
    })

class ServerConfigForm(forms.ModelForm):
    "Server config form from ServerConfig model"
    class Meta:
        "Meta class for use by django to create form"
        model = ServerConfig

urlpatterns = patterns('',
   url(r'^json/server/config',
       ServerConfigCollection(permitted_methods = ['POST', "GET"])),
   url(r'^json/merged/search/(?P<machine_name>.*)', MergedCollection()),
   url(r'^json/merged/name/(?P<machine_name>.*)$',
       MergedEntry(permitted_methods=['GET'])),
   url(r'^json/machine/search/(?P<machine_name>.*)',
       MachineCollection()),
   url(r'^json/machine/name/(?P<machine_name>.*)$',
       MachineEntry(permitted_methods=['GET', 'POST'])),
   url(r'^json/include/search/(?P<include_name>.*)', IncludeCollection()),
   url(r'^json/include/name/(?P<include_name>.*)$',
       IncludeEntry(permitted_methods=['GET', 'POST'])),
   url(r'^json/bom/search/(?P<bom_name>.*)', BomCollection()),
   url(r'^json/bom/name/(?P<bom_name>.*)$',
       BomEntry(permitted_methods=['GET', 'POST'])),
   url(r'^json/package/search/(?P<package_name>.*)', PackageCollection()),
   url(r'^json/package/name/(?P<package_name>.*)$',
       PackageEntry(permitted_methods=['GET', 'POST'])),
   url(r'^json/dist/search/(?P<dist_name>.*)', DistCollection()),
   url(r'^json/dist/name/(?P<dist_name>.*)', DistEntry()),
   url(r'^json/status/search/(?P<machine_name>.*)', StatusCollection()),
   url(r'^json/status/name/(?P<machine_name>.*)$',
       StatusEntry(permitted_methods=['GET'])),
   url(r'^json/summary/search/(?P<machine_name>.*)', StatusCollection()),
   url(r'^json/summary/name/(?P<machine_name>.*)$',
       SummaryEntry(permitted_methods=['GET'])),
   url(r'^server/config', config_setting_form),
)



"CnmResource module"
from django_restapi.resource import Resource
from configs.models import ServerConfig
import os, time
import yaml
from bombardier_server.cnm.Exceptions import InvalidServerHome, DispatcherOffline
from bombardier_server.cnm.Exceptions import DispatcherError

from Pyro.util import getPyroTraceback
from bombardier_core.static_data import OK, FAIL, SERVER_CONFIG_FILE
import Pyro.core
import StringIO
import traceback
from commands import getstatusoutput as gso
from signal import SIGTERM

DISPATCHER_INFO_FILE = "dispatcher_info.yml"

class CnmResource(Resource):
    "Base resource class"
    def __init__(self, authentication=None, permitted_methods=None,
                 mimetype=None):
        Resource.__init__(self, authentication, permitted_methods, mimetype)

    @classmethod
    def debug_log(cls, message):
        "This is for troubleshooting"
        fh = open("/tmp/web.log", 'a')
        fh.write("\nDEBUG>>> "+message+"\n")
        fh.flush()
        fh.close()

    @classmethod
    def get_server_home(cls):
        "Return server_home"
        server_home = None
        try:
            config_info = yaml.load(open(SERVER_CONFIG_FILE).read())
            server_home = config_info.get("server_home")
        except:
            raise InvalidServerHome(server_home)

        if not os.path.isdir(server_home):
            raise InvalidServerHome(server_home)

        return server_home

    @classmethod
    def get_dispatcher_info(cls):
        "read the file that the dispatcher creates when it's started"
        dispatcher_info = {}
        server_home = cls.get_server_home()
        config_file_path = os.path.join(server_home, "dispatcher_info.yml")
        if not os.path.isfile(config_file_path):
            msg = "Could not read %s (is the dispatcher started?)"
            raise CnmServerException(msg % config_file_path)
        try:
            dispatcher_info = yaml.load(open(config_file_path).read())
        except:
            msg = "Invalid data in %s (is the dispatcher started?)"
            raise CnmServerException(msg % config_file_path)
        return dispatcher_info

    @classmethod
    def get_dispatcher(cls, dispatcher_uri = None):
        "Create and return a dispatcher"
        try:
            if dispatcher_uri == None:
                config_entry = ServerConfig.objects.get(name="dispatcher_uri")
                dispatcher_uri = config_entry.value
            dispatcher = Pyro.core.getProxyForURI(dispatcher_uri)
            data = dispatcher.check_status()
        except ServerConfig.DoesNotExist:
            raise DispatcherOffline()
        except Pyro.core.ProtocolError:
            raise DispatcherOffline()
        return dispatcher

    @classmethod
    def _check_dispatcher_running(cls, uri):
        "Check for tcp port listener"
        tcp_port = uri.split(':')[-1].split('/')[0]
        cmd = 'lsof -i tcp:%s | grep python | grep -v grep' % tcp_port
        _status, output = gso(cmd)
        if output:
            return True
        return False

    @classmethod
    def _wait_for_dispatcher(cls, check_func, arg, timeout=10.0):
        "Use check_func to check for desired dispatcher state"
        start_time = time.time()
        while not check_func(arg):
            if (time.time() - start_time) > timeout:
                return FAIL
            time.sleep(0.25)
        time.sleep(1)
        return OK

    def get_dispatcher_dict(self):
        yaml_file = os.path.join(self.get_server_home(), DISPATCHER_INFO_FILE)
        if not os.path.isfile(yaml_file):
            return None
        dispatcher_info = open(yaml_file).read()
        return yaml.load(dispatcher_info)

    def attach_dispatcher(self, dispatcher_uri):
        "Connect to an already-running dispatcher"
        try:
            self.get_dispatcher(dispatcher_uri)
        except DispatcherOffline:
            raise DispatcherError("Dispatcher %s is not online")
        dispatcher_dict = self.get_dispatcher_dict()
        dispatcher_dict["dispatcher_uri"] = dispatcher_uri
        self._standard_update(dispatcher_dict)
        server_co = ServerConfig.objects.get(name="dispatcher_uri")
        return OK
        #return self._wait_for_dispatcher(self._check_dispatcher_running, server_co.value)

    @classmethod
    def dump_exception(cls, request, exception):
        "Pretty print an exception"
        exc = StringIO.StringIO()
        traceback.print_exc(file=exc)
        exc.seek(0)
        data = exc.read()
        #traceback_data = []
        #for line in data.split('\n'):
            #traceback_data.append(line)
            #ermsg = "%% %s" % line
            #print "%s : %s " % (request.user, ermsg)
        traceback_data = getPyroTraceback(exception)
        formatted_data = [ x.replace('\n', '') for x in traceback_data ]
        return {"status": FAIL, "traceback": formatted_data}

    @classmethod
    def _form_update(cls, query_dict):
        "Update or create values based on form POST."
        [ obj.delete() for obj in ServerConfig.objects.all() ]
        names = [ i for i in query_dict if i.endswith('name') ]
        for form_entry in names:
            id_num = form_entry.split('-')[1]
            name = query_dict[form_entry]
            value = query_dict['form-%s-value' % id_num]
            if name:
                server_co = ServerConfig.objects.get_or_create(id=id_num)[0]
                server_co.value = value
                server_co.name = name
                server_co.save()

    @classmethod
    def _standard_update(cls, query_dict):
        "Update or create on standard POST"
        for name in query_dict:
            value = query_dict[name]
            try:
                server_co = ServerConfig.objects.get(name=name)
            except ServerConfig.DoesNotExist:
                server_co = ServerConfig.objects.create()
                server_co.name = name
            server_co.value = value
            server_co.save()





"Bombardier Machine Interface"
#!/usr/bin/env python

import re, os, base64
import time
from commands import getstatusoutput as gso
import glob
import yaml
import simplejson as json
import StringIO
import copy
import traceback
from MachineInterface import MachineInterface, MachineUnavailableException
from Exceptions import MachineConfigurationException, BombardierMachineException
from Exceptions import MachineStatusException, PackageNotFound, RestoreFailure
from bombardier_core.static_data import OK, FAIL, EXECUTE
from bombardier_core.static_data import INIT, FIX, PURGE
from bombardier_core.static_data import BACKUP, RESTORE
from bombardier_core.static_data import ACTION_DICT, RETURN_DICT
from bombardier_core.mini_utility import update_dict
from MachineStatus import MachineStatus, LOCAL_PACKAGES
from bombardier_core.Cipher import Cipher

from pexpect import EOF
import tempfile

BLK_SIZE = 77
TMP_FILE = "tmp.yml"

class BombardierMachineInterface(MachineInterface):
    "Bombardier subclass of MachineInterface"

    def __init__(self, machine_config, server_log):
        "Set up machine interface and set up based on the environment."        
        MachineInterface.__init__(self, machine_config, server_log)
        self.status_data    = {}
        self.local_filename = ''
        self.report_info    = ''
        self.default_group  = "root"
        self.exit_code      = FAIL
        self.action_result  = []
        self.state_machine  = self._get_state_machine()
        self.machine_status = MachineStatus(self.server_home, self.machine_name)

        log_re_str = "\d+\-\d+\-\d+\s\d+\:\d+\:\d+\,\d+\|([A-Z]+)\|(.+)\|" 
        self.log_matcher = re.compile(log_re_str) 

        self.trace_matcher = re.compile( "\|\|\>\>\>(.+)" )
        if self.platform.startswith('win'):
            self.python  = '/cygdrive/c/Python25/python.exe'
            self.spkg_dir = '/cygdrive/c/spkg'
        else:
            self.python  = '/usr/bin/python'
            self.spkg_dir = '/opt/spkg'
        if self.data.get("python_path"):
            self.python = self.data.get("python_path")
        if self.data.get("spkg_path"):
            self.spkg_dir = self.data.get("spkg_path")
        self.pull_report = True

    ##############################################################
    ## PRIVATE METHODS
    ##############################################################

    def _get_state_machine(self):
        "Return a list of compiled regular expression objects for log handling"
        exit_re        = re.compile("\=\=EXIT-CODE\=\=:(\d+)")
        report_re      = re.compile("\=\=REPORT\=\=:(.*)")
        output_re      = re.compile("\=\=OUTPUT\=\=:(.*)")
        no_report_re   = re.compile("Unable to shred before deleting")
        uninstall_re   = re.compile("Uninstalling package \((\S+)\)")
        install_re     = re.compile("Beginning installation of \((\S+)\)")
        result_re      = re.compile("(\S+) result for (\S+) : (\d)")

        state_machine = [ [exit_re,        self._get_exit_code],
                          [report_re,      self._get_report],
                          [output_re,      self._get_output],
                          [no_report_re,   self._no_report],
                          [uninstall_re,   self._uninstall],
                          [install_re,     self._install],
                          [result_re,      self._get_action_result],
                        ]
        return state_machine

    # STATE-MACHINE ROUTINES

    def _action_start(self, action, package_name):
        "Logging for beginning of an action"
        message = "%s installing %s" % (self.machine_name, package_name)
        self.polling_log.info(message)
        self.server_log.info(message, self.machine_name)

    def _get_exit_code(self, exit_code):
        "Parse action code from string data, this needs to be more defensive."
        message = "Output received: %s" % exit_code
        self.polling_log.debug(message)
        self.server_log.debug(message, self.machine_name)
        self.exit_code = int(exit_code)

    def _get_report(self, yaml_line):
        "Add a yaml line to the report"
        self.report_info += yaml_line + "\n"

    def _get_output(self, output):
        "Add output to logging and action result"
        #self.server_log.info(output)
        #self.polling_log.info(output, self.machine_name)
        self.action_result.append(output)

    def _no_report(self, data):
        "Don't pull report"
        self.pull_report = False

    def _uninstall(self, package_name):
        "Flag the logger that an un-installation is taking place"
        self._action_start("uninstalling", package_name)

    def _install(self, package_name):
        "Flag the logger that an installation is taking place"
        self._action_start("installing", package_name)

    def _get_action_result(self, data):
        "Add action result to logs"
        action, package_name, result = data
        message = "%s %s: %s" % (action.lower(),
                    RETURN_DICT[int(result)], package_name)
        self.polling_log.info(message)
        self.server_log.info(message, self.machine_name)
        self.action_result.append(message)

    # END STATE MACHINE ROUTINES

    def _stream_file(self, file_name):
        "Stream a file"
        plain_text = open(file_name, 'rb').read()
        return self._stream_data(plain_text)

    def _stream_data(self, plain_text):
        "Send file contents over stdin via pxssh"
        import zlib
        #self.server_log.info("Pre stream", self.machine_name) 
        if self.platform.startswith("win"):
            yaml_loaded = yaml.load(plain_text)
            plain_text = json.dumps(yaml_loaded)
            encoded = base64.encodestring(plain_text)
            #re_encoded = []
            #for line in encoded.split('\n'):
                #re_encoded.append(line)
                #self.server_log.info('[%s]' % line, self.machine_name)
            #encoded = '\n'.join(re_encoded) + '\n'
            #self.server_log.info("Last character in encoded: [%s][%d]" % (encoded[-2], int(encoded[-2])), self.machine_name)
            #encoded += '-\n'
        else:
            #compressed = zlib.compress(plain_text)
            #encoded = base64.encodestring(compressed)
            self.server_log.info("NOT COMPRESSING", self.machine_name)
            yaml_loaded = yaml.load(plain_text)
            plain_text = json.dumps(yaml_loaded)
            encoded = base64.encodestring(plain_text)
        self.ssh_conn.setecho(False)
        handle = StringIO.StringIO(encoded)
        while True:
            response = self.ssh_conn.read(2)
            #self.server_log.info("Read (%s)" % response)
            chunk = handle.read(BLK_SIZE)
            if chunk == '':
                chunk = ' '*(BLK_SIZE)+'\n'
                #self.server_log.info("* SENDING: (%s)[%d]" % (chunk, len(chunk)), self.machine_name) 
                self.ssh_conn.send(chunk)
                break
            if len(chunk) < BLK_SIZE:
                if self.platform.startswith("win"):
                    pad = ' '*(BLK_SIZE-len(chunk)-1)
                    chunk = chunk[:-1] + '-' + pad + '\n'
                else:
                    pad = ' '*(BLK_SIZE-len(chunk))
                    chunk = chunk[:-1] + pad + '\n'
            #self.server_log.info("  SENDING: (%s)[%d]" % (chunk, len(chunk)), self.machine_name) 
            self.ssh_conn.send(chunk)

    def _scp_all_client_data(self, raw_data):
        """If the remote machine has a configuration key, we will
        encrypt its configuration data and secure-copy it to him rather
        than stream the data to bc.py's stdin"""
        enc_key = self.data['config_key']
        yaml_data_str = yaml.dump(raw_data)
        cipher = Cipher(enc_key)
        enc_data = cipher.encrypt_string(yaml_data_str)
        dest_path = os.path.join(self.spkg_dir, self.machine_name, "client.yml.enc")
        temp_file = tempfile.mkstemp()[1]
        open(temp_file, 'w').write(enc_data)
        self.scp(temp_file, dest_path, False)
        #self.server_log.debug("Cleaning local temporary file %s" % temp_file)
        #os.system("rm -f %s" % temp_file)
        self._stream_data("config_key: '%s'\n" % enc_key)

    def _get_all_client_data(self):
        """For a remote machine that doesn't have a configuration key, this
        method gathers all the configuration data that it needs"""
        send_data = {"config_data": self.data, 
                     "package_data": {},
                    }
        package_names = self.machine_status.get_all_package_names(self.data.get("packages"))
        for package_name in package_names:
            this_package_data = self.machine_status.get_package_data(package_name)
            if not this_package_data:
                message = "Could not find package data for %s."
                self.polling_log.error(message % package_name)
                raise MachineConfigurationException(self.machine_name)
            send_data["package_data"][package_name] = this_package_data
        return send_data

    def _send_all_client_data(self, action):
        "Stream package metadata to a remote client"
        if action == PURGE:
            send_data = {"config_data": self.data}
        else:
            send_data = self._get_all_client_data()
        if 'config_key' in self.data and action != INIT:
            self._scp_all_client_data(send_data)
        else:
            self.polling_log.info("Streaming configuration data...")
            self._stream_data(yaml.dump(send_data))

    def _get_bc_command(self):
        "Get shell command to run bc.py on the target system."
        cmd = ""
        if self.platform.startswith("win"):
            cmd = 'ipy c:\\\\spkg\\\\bc.py '
        else:
            cmd = 'bc.py'
        return cmd

    def _run_bc(self, action, package_name, script_name, arguments, debug):
        "Run bc.py on a remote machine, and watch the logs"
        self.report_info = ''
        self.chdir(self.spkg_dir)

        cmd = self._get_bc_command()
        args_str = ' '.join(arguments)
        cmd += " {0} {1} {2} {3} {4}".format(ACTION_DICT[action],
               self.machine_name, package_name, script_name, args_str)
        self.ssh_conn.sendline(cmd)
        self._send_all_client_data(action)
        self._watch_bc()

        try:
            if self.report_info:
                return yaml.load(self.report_info)
            return self.action_result
        except Exception:
            msg = "Cannot read report data: (%s)" % self.report_info
            self.polling_log.error(msg)
            self.exit_code = FAIL
            return [msg]

    def _process_message(self, message):
        "Parse log message and possibly take action"
        for state in self.state_machine:
            match, function = state
            grep_info = match.findall(message)
            if grep_info:
                if function:
                    function(grep_info[0])
                    return True
        return False

    def _watch_bc(self):
        "Watch log output from bc.py"
        while True:
            expect_list = [self.ssh_conn.PROMPT, self.trace_matcher,
                           self.log_matcher]
            found_index = self.ssh_conn.expect(expect_list, timeout=6000)
            if found_index == 0: # BC exited
                if self.ssh_conn.before.strip():
                    msg = "Remaining output: %s" % self.ssh_conn.before.strip()
                    self.polling_log.debug(msg)
                    self.server_log.debug(msg, self.machine_name)
                self.ssh_conn.setecho(False)
                break
            elif found_index == 1: # Stack trace
                raise BombardierMachineException()
            elif found_index == 2: # Log message
                level, message = self.ssh_conn.match.groups()
                if not self._process_message(message):
                    message = message.strip()
                    self.polling_log.log_message(level, message)
                    self.server_log.log_message(level, message, self.machine_name)

    def _check_file(self, package_name, file_path):
        "Determine if a file specified in the type-5 package actually exists"
        if file_path:
            if not os.path.isfile(file_path):
                msg = "%s not found." % file_path
                self.server_log.error(msg, self.machine_name)
                raise PackageNotFound(package_name, file_path)

    def _get_type_5_files(self, package_name, package_data):
        """Examine the files specified by a type-5 package and create
        a dictionary of files to be synchronized to the remote machine"""
        package_sync = {
                        "injectors": [],
                        "libs": [],
                       }

        for dir_name in [ "libs", "injectors" ]:
            sync_section = package_data.get(dir_name, [])
            for sync_item in sync_section:
                sync_data = sync_section[sync_item]
                sync_file = sync_data.get("path")
                if not sync_file:
                    raise PackageNotFound(package_name, "No PATH DEFINED")
                if not sync_file.startswith(os.path.sep):
                    sync_file = os.path.join(self.server_home, "repos",
                                             dir_name, sync_file)
                self._check_file( package_name, sync_file )
                package_sync[dir_name].append(sync_file)
        return package_sync

    def _create_sync_directory(self, files_to_send):
        "Creating a temporary directory where RSYNC can operate"
        tmp_path = tempfile.mkdtemp()
        for directory_name in files_to_send:
            subdir = os.path.join(tmp_path, directory_name)
            cmd = "mkdir -p %s" % subdir
            os.system(cmd)
            file_list = files_to_send[directory_name]
            for file_name in file_list:
                cmd = "ln -s %s %s" % (file_name, subdir)
                self.server_log.info("Running: %s" % cmd)
                os.system(cmd)
        return tmp_path

    def _rsync_repository(self, tmp_path, dest):
        """Performing an RSYNC command to get all repos files to
        the remote machine"""
        cmd = "rsync -La %s %s@%s:%s" 
        cmd = cmd % (tmp_path, self.username, self.ip_address, dest)
        self.server_log.info("Running: %s" % cmd)
        files = glob.glob("%s/*" % tmp_path)
        self.polling_log.debug(cmd)
        status, output = gso(cmd)
        if status != OK:
            msg = "Error rsyncing repository: %s" % output
            raise MachineUnavailableException(self.machine_name, msg)

    def _get_delivered_packages(self):
        "Find out what packages exist on the remote machine"
        try:
            package_names = self.machine_status.get_package_names_from_progress()
            delivered_packages = package_names.get(LOCAL_PACKAGES, [])
        except MachineStatusException:
            msg = "No/invalid status data. Assuming there are "\
                  "no packages on remote system"
            self.polling_log.warning(msg)
            delivered_packages = []
        return delivered_packages

    def _get_files_to_send(self, required_base_names, delivered_packages):
        "Determine what files need to be sync'd to the remote machine"
        files_to_send = {"type4": []}
        for base_name in required_base_names:
            newest_data = self.machine_status.get_package_data(base_name)
            version = newest_data.get("package-version")
            if version == 4:
                newest_name = newest_data.get("install", {}).get("fullName")
                path_to_file = os.path.join(self.server_home, "repos",
                                            "type4", "%s.spkg" % newest_name)
                self._check_file(base_name, path_to_file)
                if newest_name not in delivered_packages:
                    files_to_send["type4"].append(path_to_file)
            elif version == 5:
                package_sync = self._get_type_5_files(base_name, newest_data)
                files_to_send = update_dict(files_to_send, package_sync)
        return files_to_send

    def _upload_new_packages(self):
        "Send needed packages to a machine using symlinks and rsync"
        self.polling_log.info("Syncing packages...")
        delivered_packages = self._get_delivered_packages()
        required_base_names = copy.deepcopy(self.data.get("packages"))
        files_to_send = self._get_files_to_send(required_base_names,
                                                delivered_packages)

        tmp_path = self._create_sync_directory(files_to_send)
        dest = os.path.join(self.spkg_dir, "repos")
        self._rsync_repository(tmp_path + os.path.sep, dest)
        dest = os.path.join(self.spkg_dir, self.machine_name, "packages")

        type_4_dir = os.path.join(self.spkg_dir, "repos", "type4")
        spkg_glob = "%s/*.spkg" % type_4_dir
        cmd = "[ -e %s ] && ln -fs %s %s/" % (spkg_glob, spkg_glob, dest)
        self.ssh_conn.sendline(cmd)
        if not self.ssh_conn.prompt(timeout = 5):
            dead = True
            msg = "Unable to create links: %s" % self.ssh_conn.before
            raise MachineUnavailableException(self.machine_name, msg)
            
        self.server_log.debug("Removing directory %s..." % tmp_path)
        os.system("rm -rf %s" % tmp_path)
        self.polling_log.debug("...Finished syncing packages.")

    def _dump_trace(self):
        "Pretty print a stack trace into the logs"
        stack_trace = []
        found_index = 1
        while found_index == 1:
            stack_trace.append(self.ssh_conn.match.groups()[0])
            expect_list = [self.ssh_conn.PROMPT, self.trace_matcher,
                           self.log_matcher]
            found_index = self.ssh_conn.expect(expect_list, timeout=6000)
        t_string = ''.join(stack_trace)

        noop_re = "NoOptionError\: No option \'(\w+)\' in section\: \'(\w+)\'"
        re_obj = re.compile(noop_re)
        data = re_obj.findall(t_string)
        invalid_data_msg = "Invalid client configuration data"
        if data:
            self.polling_log.error(invalid_data_msg)
            self.server_log.error(invalid_data_msg, self.machine_name)
            if len(data) == 2:
                need_msg = "Need option '%s' in section '%s'." % \
                            (data[0], data[1])
            else:
                need_msg = "Need options: %s" % data
            self.polling_log.info(need_msg)
            self.server_log.error(need_msg, self.machine_name)
        no_section_re = re.compile("NoSectionError\: No section\: \'(\w+)\'")
        data = no_section_re.findall(t_string)
        if data:
            self.polling_log.error(invalid_data_msg)
            self.server_log.error(invalid_data_msg, self.machine_name)
            need_msg = "Need section '%s'." % (data[0])
            self.polling_log.info(need_msg)
            self.server_log.error(need_msg, self.machine_name)
        else:
            for line in stack_trace:
                self.polling_log.error(line)
                self.server_log.error(line, self.machine_name)

    def _clear_script_output(self, script_name):
        "Remove old output yml file if it exists on remote machine"
        file_name = "%s-%s.yml" % (self.machine_name, script_name)
        file_path = os.path.join(self.server_home, "output", file_name)
        if os.path.isfile(file_path):
            os.unlink(file_path)
        return

    def _get_status_yml(self):
        "Pull status.yml from remote machine and cache it locally"
        status_dir = os.path.join(self.server_home, 'status')
        if not os.path.isdir( status_dir ):
            os.makedirs( status_dir )

        remote_file = "%s/%s/status.yml" % (self.spkg_dir, self.machine_name)
        status_file = os.path.join(status_dir, "%s.yml" % self.machine_name)
        self.get(remote_file, status_file)
        return
        #new_line = 'cat %s/%s/status.yml;echo "======="'
        #self.ssh_conn.sendline(new_line % (self.spkg_dir, self.machine_name))
        #self.ssh_conn.prompt()
        #status_yml = str(self.ssh_conn.before).split("======")[0]
        #status_yml = status_yml.replace('\r','')
        #for line in status_yml.split('\n'):
            #self.polling_log.info("STATUS [%s]" % line)
        #try:
            #yaml.load(status_yml)
        #except:
            #error_file =  os.path.join(status_dir, "error.yml")
            #msg = "status.yml could not be parsed (writing to %s)" % error_file
            #self.polling_log.error(msg)
            #open( error_file, "w" ).write(status_yml)
            #return
        #status_file = os.path.join(status_dir, "%s.yml" % self.machine_name)
        #try:
            #open( status_file, 'w' ).write(status_yml)
            #cmd = "chgrp %s %s 2> /dev/null"
            #os.system(cmd % (self.default_group, status_file))
            #cmd = "chmod 660 %s 2> /dev/null"
            #os.system(cmd % (status_file))
        #except IOError, ioe:
            #msg = "Unable to write '%s' (%s)" % (status_file, ioe)
            #self.polling_log.error(msg)

    def _process_backup(self, backup_dict, package_name):
        '''
        We've just instructed a package to back itself up. We need to haul the
        backup data back to archive it and collect some statistics on it.
        backup_dict -- Detailed information about the backup
            Example: {'__BACKUP_DIR__': '/tmp/tmp3GHMRw',
                      '__POST_BACKUP__': 'start',
                      '__PRE_BACKUP__': 'stop',
                      '__START_TIME__': 1284766556.062264,
                      'main_file': {'status': 0,
                                    'elapsed_time': 0.0013320446014404297,
                                    'file_name': '/tmp/foogazi/test_type_5',
                                    'size': 18L,
                                    'md5': {'test_type_5': 'a670f8128b02d00725cff2b4973fb47e',
                                            'test_type_5.bz2': '17e6a319e8e0b496dab7eb9b3ce03994'
                                           }
                                   }
                     }
        '''
        remote_dir = backup_dict.get("__BACKUP_DIR__")
        if not remote_dir:
            msg = "Package does not request data be copied back from machine."
            self.polling_log.warning(msg)
            self.server_log.warning(msg)
            return backup_dict
        start_time = str(int(backup_dict.get("__START_TIME__")))
        archive_dir = os.path.join(self.server_home, "archive",
                                   self.machine_name, package_name, start_time)
        os.system("mkdir -p %s" % archive_dir)
        cmd = "rsync -La %s@%s:%s/* %s" 
        cmd = cmd % (self.username, self.ip_address, remote_dir, archive_dir)
        self.polling_log.debug(cmd)
        status, output = gso(cmd)
        if status != OK:
            msg = "Error rsyncing remote backup: %s" % output
            backup_dict["__SYNC__"] = FAIL
            self.status = FAIL
        else:
            backup_dict["__SYNC__"] = OK
            self.run_cmd("rm -rf %s" % remote_dir)
            del backup_dict["__BACKUP_DIR__"]
            backup_summary = yaml.dump(backup_dict)
            summary_file = os.path.join(archive_dir, "backup_info.yml")
            open(summary_file, 'w').write(backup_summary)
        return backup_dict

    ##############################################################
    ## PUBLIC METHODS
    ##############################################################

    def freshen(self, job_name, require_status):
        "Refresh config data"
        self.set_job(job_name)
        try:
            MachineInterface.freshen(self)
        except MachineUnavailableException:
            self.unset_job()
            raise
        try:
            self.machine_status.freshen()
        except MachineStatusException:
            msg = "Can't find status. Fetching from machine."
            self.polling_log.warning(msg)
            self._get_status_yml()
            try:
                self.machine_status.freshen()
            except MachineStatusException:
                if require_status:
                    self.unset_job()
                    raise
        return OK

    def _sync_restore_data(self, package_name, target_list):
        '''
        Prior to a restore command, sends all necessary data to the remote
        machine to $SPKG_DIR/archive
        package_name -- name of package being restored
        target_list -- list of (one) name of a backup dataset
        '''
        if len(target_list) != 1:
            msg = "One and only one Restore Target can be specified (received %s)"
            raise RestoreFailure(msg % target_list)
        target = target_list[0]
        remote_dir = os.path.join(self.spkg_dir, "archive", package_name, target)
        self.run_cmd(" mkdir -p %s" % remote_dir)
        archive_dir = os.path.join(self.server_home, "archive",
                                   self.machine_name, package_name, target)
        if not os.path.isdir(archive_dir):
            reason = "{0} does not exist".format(archive_dir)
            errmsg = "Cannot restore target {0} for package {1}/machine {2}"\
                     ": {3}"
            errmsg = errmsg.format(target, package_name, self.machine_name, reason)
            self.polling_log.error(errmsg)
            raise RestoreFailure(reason, target, package_name, self.machine_name)
            
        cmd = "rsync -La {0}/* {1}@{2}:{3}" 
        cmd = cmd.format(archive_dir, self.username, self.ip_address,
                         remote_dir)
        self.server_log.info("Running: %s" % cmd)
        self.polling_log.debug(cmd)
        self.polling_log.info("Copying archive to remote machine...")
        status, output = gso(cmd)
        if status != OK:
            msg = "Error rsyncing restore data: %s" % output
            raise RestoreFailure(msg)

    def take_action(self, action, package_name, script_name, arguments, debug):
        '''
        Run a package-oriented action on a remote machine
        action -- one of the following: EXECUTE, INIT, INSTALL,
                  UNINSTALL, VERIFY, CONFIGURE, DRY_RUN, BACKUP
        package_name -- the name of a package to operate on
        script_name -- the name of a script to run (againsta a package)
        debug -- defunct
        '''
        from bombardier_core.static_data import ACTION_REVERSE_LOOKUP
        self.server_log.info("test", "test")
        mss = "Action: %s (%s)" % (ACTION_REVERSE_LOOKUP[action], action)
        self.polling_log.info(mss)

        message = []
        start_time = time.time()
        try:
            self.action_result = []
            self.pull_report = True
            if action in [ EXECUTE, BACKUP ]:
                self._clear_script_output(script_name)
            if action == RESTORE:
                self._sync_restore_data(package_name, arguments)
            if not action in [ INIT, FIX, PURGE ]:
                self._upload_new_packages()
            message = self._run_bc(action, package_name, script_name,
                                   arguments, debug)
        except MachineUnavailableException:
            message = ["Remote system refused connection."]
            self.exit_code = FAIL
        except MachineConfigurationException, exc:
            self.exit_code = FAIL
            message = ["Machine configuration error: %s" % exc]
        except EOF:
            self.exit_code = FAIL
            message = ["Machine unexpectedly disconnected."]
        except BombardierMachineException, exc:
            self.server_log.error("FOUND A STACK TRACE")
            self._dump_trace()
            self.ssh_conn.prompt()
            self._get_status_yml()
            self.exit_code = FAIL
            message = ["Machine raised an exception."]
        except Exception, exc:
            print "------------------- PROCESS EXCEPTION"
            exc = StringIO.StringIO()
            traceback.print_exc(file=exc)
            exc.seek(0)
            data = exc.read()
            ermsg = ''
            for line in data.split('\n'):
                ermsg = "%% %s" % line
                self.polling_log.error(ermsg)
                self.server_log.error(ermsg, self.machine_name)
            self.exit_code = FAIL
            message = ["Exception in client-handling code."]
        self._get_status_yml()
        msg = "take_action is returning: %s / %s" % (self.exit_code, message)
        self.server_log.debug(msg)
        if action == BACKUP:
            if self.exit_code == OK:
                if type(message) != type({}):
                    errmsg = "Backup returned illegal data: %s" % message
                    self.server_log.error(errmsg, self.machine_name)
                    self.exit_code = FAIL
                else:
                    message["__START_TIME__"] = start_time
                    message = self._process_backup(message, package_name)
        return self.exit_code, message




#!/usr/bin/env python

"sets up one of the bombardier components"

from distutils.core import Command, setup
from unittest import TextTestRunner, TestLoader
from os.path import splitext, basename, join as pjoin, walk
import os
from distutils.core import setup
from lib._version import version_info
import glob
import sys

MANIFEST_EXTRA = "MANIFEST.in.extra"

def is_unicode(val):
    return type(val) == type(u"unicode")

def is_list(val):
    return type(val) == type([])

def is_set(val):
    return type(val) == type(set([]))

def is_dict(val):
    return type(val) == type({})

def uni_dict_to_str_dict(uni_dict):
    new_dict = {}
    for key in uni_dict:
        item = uni_dict[key]
        new_item = uni_to_str(item)
        new_dict[str(key)] = new_item
    return new_dict

def uni_list_to_str(uni_list):
    new_list = []
    for val in uni_list:
        new_item = uni_to_str(val)
        new_list.append(new_item)
    return new_list

def uni_to_str(val):
    new_val = None
    if is_dict(val):
        new_val = uni_dict_to_str_dict(val)
    elif is_list(val):
        new_val =  uni_list_to_str(val)
    elif is_set(val):
        new_val =  set(uni_list_to_str(list(val)))
    elif is_unicode(val):
        new_val = str(val)
    else:
        new_val = val
    return new_val


class CleanCommand(Command):
    user_options = [ ]
    def initialize_options(self):
        self._clean_me = [ ]
        for root, dirs, files in os.walk('.'):
            for f in files:
                if f.endswith('.pyc'):
                    self._clean_me.append(pjoin(root, f))

    def finalize_options(self):
        pass

    def run(self):
        for clean_me in self._clean_me:
            try:
                os.unlink(clean_me)
                print "%s deleted." % clean_me
            except:
                pass

class TestCommand(Command):
    user_options = [ ]
    def initialize_options(self):
        self._dir = os.getcwd()

    def finalize_options(self):
        pass

    def run(self):
        '''
        Finds all the tests modules in tests/, and runs them.
        '''
        testfiles = [ ]
        for t in glob.glob(pjoin(self._dir, 'tests', '*.py')):
            if not t.endswith('__init__.py'):
                testfiles.append('.'.join(
                    ['tests', splitext(basename(t))[0]])
                )

        print "TEST FILES:",testfiles
        tests = TestLoader().loadTestsFromNames(testfiles)
        t = TextTestRunner(verbosity = 1)
        t.run(tests)

def get_data(file_stub_name):
    if sys.platform == "cli":
        import simplejson as json
        file_name = file_stub_name + ".json"
        data = json.loads(open(file_name).read())
        return uni_to_str(data)
    else:
        import yaml
        file_name = file_stub_name + ".yml"
        return yaml.load(open(file_name).read())

def main(package_data):
    cmdclasses = {}
    # 'test' is the parameter as it gets added to setup.py

    setup(
          cmdclass = { 'test': TestCommand, 'clean': CleanCommand },
          name = package_data["name"],
          description = package_data["description"],
          version = "%(branch_nick)s-%(revno)d" % version_info,
          packages = package_data["packages"],
          package_data = package_data.get("package_data", {}),
          package_dir = package_data["package_dir"],
          data_files = package_data.get("data_files"),
          author = package_data["author"],
          author_email = package_data["author_email"],
          long_description = package_data["long_description"],
          scripts = package_data.get("scripts"),
          install_requires = package_data["install_requires"],
          provides = package_data["provides"],
          url = package_data["url"],
          classifiers = package_data["classifiers"],
         )

if __name__ == "__main__":
    package_data = get_data("project_info")
    component_data = get_data("component_info")
    if os.path.isfile(MANIFEST_EXTRA):
        append_manifest_data = open(MANIFEST_EXTRA).read()
        if append_manifest_data:
            open("MANIFEST.in", 'a').write(append_manifest_data)
    package_data.update(component_data)
    main(package_data)



#!/usr/bin/env python
"LocalInterface: Interface with the local server"

from bombardier_core.static_data import OK
from AbstractMachineInterface import AbstractMachineInterface
from Exceptions import CnmServerException, BuildError
import pexpect
import tempfile
import os, re, sys
import yaml, random
import StringIO, traceback
from commands import getstatusoutput as gso
from MockConfig import MockConfig

BUILD_BASE_DIR = "/tmp/bdr_build"

class LocalMachineInterface(AbstractMachineInterface):
    "Interface to a machine"
    def __init__(self, machine_config, server_log):
        AbstractMachineInterface.__init__(self, machine_config, server_log)

    def terminate(self):
        "Part of interface. No function for local system"
        pass

    def chdir(self, path):
        "Part of interface. No function for local system"
        pass

    def gso(self, cmd, raise_on_error=True, cmd_debug=False):
        "Part of interface. No function for local system"
        pass

    def run_cmd(self, command_string):
        "Run a remote shell command"
        return [OK, []]

    def _dump_trace(self):
        "Pretty print a stack trace into the logs"
        exc = StringIO.StringIO()
        traceback.print_exc(file=exc)
        exc.seek(0)
        data = exc.read()
        ermsg = ''
        for line in data.split('\n'):
            ermsg = "%% %s" % line
            self.polling_log.error(ermsg)
            self.server_log.error(ermsg, self.machine_name)

    def _svn_checkout(self, version, svn_user, svn_password,
                      svn_url, checkout_dir, debug):
        "Check out some SVN code for building a package"
        command = "export"
        if debug:
            command = "co"
        username_section = ''
        if svn_user:
            username_section = ' --username %s ' % svn_user
        cmd = "svn %s --no-auth-cache -r %s %s %s %s"
        cmd = cmd % (command, version, username_section, svn_url, checkout_dir)
        #self.polling_log.info("CMD: --- %s" % cmd)
        self.server_log.info("cmd: (%s)" % cmd, self.machine_name)
        svn_conn = pexpect.spawn(cmd, timeout=30)
        output = []
        while True:
            expect_list = [re.compile("Exported revision (\d+)"),
                           re.compile("Checked out revision (\d+)"),
                           str("Password for '%s':" % svn_user),
                           re.compile("A\s+(\S+)"),
                           "svn: REPORT request failed", 
                           "is already a working copy",
                           "doesn't exist", pexpect.TIMEOUT, pexpect.EOF]
            select_index = svn_conn.expect(expect_list, timeout=50)
            #self.server_log.info("SELECT INDEX: (%s)" % select_index, self.machine_name)
            if select_index == 0 or select_index == 1:
                version = svn_conn.match.groups()[0]
                self.server_log.info("Exported version %s" % version)
                break
            elif select_index == 2:
                svn_conn.sendline(svn_password)
            elif select_index == 3:
                self.polling_log.info("--- %s" % svn_conn.match.groups()[0])
            elif select_index > 3:
                for line in svn_conn.before.split('\n'):
                    self.polling_log.info("--- %s" % line)
                    self.server_log.info("--- %s" % line, self.machine_name)
                msg = "Could not execute SVN export (%s)" % select_index
                raise CnmServerException(msg)
        svn_conn.close()
        return version

    def _build_component(self, component_dict, section_name,
                         svn_user, svn_password, tmp_path, debug, prepare):
        '''Export one item from SVN and create an appropriate TAR file
        component_dict -- dictionary of item data
        section_name -- either 'libs' or 'injectors'
        svn_user -- account to use for logging in to svn
        svn_password -- password for said account
        '''
        tar_file_path = component_dict.get("path", '')
        svn_url = component_dict["svn"]
        if svn_url.endswith('/'):
            svn_url = svn_url[:-1]
        base = svn_url.split('/')[-1]
        if prepare:
            version = "HEAD"
        else:
            version = component_dict.get("version", "HEAD")
        expected_file_path = os.path.join(self.server_home, "repos",
                                          section_name,
                                          "%s-%s.tar.gz" % (base, version))
        
        if tar_file_path == expected_file_path:
            if os.path.isfile(tar_file_path):
                msg = "No need to build %s: already exists" % base
                self.polling_log.info(msg)
                return None
        start_path = os.getcwd()
        checkout_dir = os.path.join(tmp_path, section_name, base)
        if not os.path.isdir(checkout_dir):
            os.system("mkdir -p %s" % checkout_dir)
        
        msg = "Checking out and building %s..." % base
        self.polling_log.info(msg)
        version = self._svn_checkout(version, svn_user, svn_password,
                           svn_url, checkout_dir, debug)
        os.chdir( checkout_dir )
        section_dir =  os.path.join(self.server_home, "repos", section_name)
        if not os.path.isdir(section_dir):
            status = os.system("mkdir -p %s" % section_dir)
            if status != OK:
                msg = "Could not create required directory: %s" % section_dir
                raise CnmServerException(msg)
            else:
                msg = "Created required directory: %s" % section_dir
                self.server_log.warning(msg)
        tar_file_path = os.path.join(section_dir, 
                                     "%s-%s.tar.gz" % (base, version))
        cmd = "tar -czf %s *" % ( tar_file_path )
        status, output = gso(cmd)
        if status != OK:
            self.server_log.error("Command: %s" % cmd)
            self.server_log.error("Current Directory: %s" % os.getcwd())
            self.server_log.error("Output from command: %s" % output)
            msg = "Could not create tarball"
            os.chdir( start_path )
            raise CnmServerException(msg)
        os.chdir( start_path )
        return tar_file_path, version

    def get_non_builtin_modules(self):
        modules = []
        module_list = sys.modules.values()
        for item in module_list:
            work_str = str(item)[8:-1]
            if not " from " in work_str:
                continue
            module_name = item.__name__
            item_str, source = work_str.split(" from ")
            source_path = source[1:-1]
            modules.append((item_str, source_path, module_name))
        return modules

    def remove_test_modules(self, prefix=BUILD_BASE_DIR):
        "removes any trace of modules we've imported"
        modules = self.get_non_builtin_modules()
        for item_str, source_path, module_name in modules:
            if source_path.startswith(prefix):
                self.server_log.info("UN-IMPORTING: %30s %20s / %s" % (item_str, source_path, module_name))
                del sys.modules[module_name]

    def verify_test_modules(self, prefix):
        "Assures that all modules we're importing come from what we want"
        modules = self.get_non_builtin_modules()
        for item_str, source_path, module_name in modules:
            if source_path.startswith(BUILD_BASE_DIR):
                if not source_path.startswith(prefix):
                    raise BuildError("We have a dirty build environment (%s)" % source_path)

    def pre_cleanup(self, tmp_path):
        "Make sure the build environment is clean."
        self.remove_test_modules()
        self.verify_test_modules(tmp_path)
        bad_path_entries = []
        for entry in sys.path:
            if entry.startswith(BUILD_BASE_DIR):
                bad_path_entries.append(entry)
        for bad_path_entry in bad_path_entries:
            self.polling_log.warning("Removing stale import path %s" % bad_path_entry)
            sys.path.remove(bad_path_entry) 
        self.polling_log.info("Build environment is clean.")

    def _inspect_libraries(self, tmp_path, pkg_data):
        """This will instantiate the Bombardier installer script and collect
        all of the configuration information that it requires"""
        start_path = os.getcwd()
        injectors_path = os.path.join(tmp_path, "injectors")
        if not os.path.isdir(injectors_path):
            os.system("mkdir -p %s" % injectors_path)
        os.chdir(injectors_path)
        base_name = ''
        class_name = ''
        description = ''
        self.pre_cleanup(tmp_path)
        try:
            tmp_libs = os.path.join(tmp_path, "libs")
            sys.path.append(tmp_libs)

            output = []
            metadata = {}

            class_name = pkg_data.get("class_name")
            base_name  = class_name.split('.')[0]

            letters = [ chr( x ) for x in range(65, 91) ]
            random.shuffle(letters)
            rand_name = ''.join(letters)
            self.polling_log.info("My current directory: %s" % os.getcwd())
            if base_name in sys.modules:
                sys.modules.pop(base_name)
            if class_name in sys.modules:
                sys.modules.pop(class_name)
            cmd = "import %s as %s" % ( class_name, rand_name )
            exec(cmd)
            exec("description = %s.__doc__" % (rand_name))
            if not description:
                exec("description = %s.%s.__doc__" % (rand_name, base_name))
            if description:
                self.polling_log.info("Description: %s" % description)
            else:
                self.polling_log.warning("Class %s has no description" % class_name)

            self.verify_test_modules(tmp_path)

            config = MockConfig()
            exec ('object = %s.%s(config)' % ( rand_name, base_name))

            if hasattr(object, "metadata"):
                metadata = object.metadata
            config_requests = config.get_requests()
            for item in config_requests:
                self.polling_log.info("Configuration dictionary: %s" % item)
            public_methods = {}
            for method in dir(object):
                if callable(getattr(object, method)) and not method.startswith('_'):
                    exec("docstring = object.%s.__doc__" % method)
                    if not docstring:
                        docstring = 'Undocumented Executable'
                    public_methods[method] = docstring
        except Exception, exp:
            os.chdir(start_path)
            raise

        # cleanup:
        self.remove_test_modules(tmp_path)
        sys.path.remove(tmp_libs)
        os.chdir(start_path)
        return description, config_requests, public_methods

        
    def build_components(self, package_name, svn_user, svn_password,
                         debug, prepare):
        '''We want to create files that match all of the svn entries for
        a given package
        package_name -- the name of the package we're modifying
        svn_user / svn_password -- svn credentials
        '''
        try:
            os.system("mkdir -p %s" % BUILD_BASE_DIR)
            tmp_path = tempfile.mkdtemp(dir=BUILD_BASE_DIR)
            package_file = os.path.join(self.server_home, "package",
                                        "%s.yml" % package_name)
            pkg_data = yaml.load(open(package_file).read())
            output = []
            for section_name in ["injectors", "libs"]:
                for component_name in pkg_data[section_name]:
                    component_dict = pkg_data[section_name][component_name]
                    if "svn" in component_dict:
                        tar_file_path, version = self._build_component(component_dict,
                                                              section_name,
                                                              svn_user,
                                                              svn_password,
                                                              tmp_path, debug,
                                                              prepare)
                        pkg_data[section_name][component_name]["version"] = version
                        if tar_file_path:
                            pkg_data[section_name][component_name]["path"] = tar_file_path
                            output.append("built %s" % tar_file_path)

            pkg_data["release"] = pkg_data.get("release", 0) + 1
            description, configuration, methods = self._inspect_libraries(tmp_path, pkg_data)
            pkg_data['description'] = description
            pkg_data['configuration'] = configuration
            pkg_data['executables'] = methods
            open(package_file, 'w').write(yaml.dump(pkg_data))
            self.polling_log.info("Note: not deleting %s" % tmp_path)
        except Exception:
            self._dump_trace()
            advice = "%s is still available for code inspection." % tmp_path
            self.polling_log.error(advice)
        
        return OK, output




import os
from bombardier_core.static_data import SERVER_CONFIG_FILE
SERVER_CONFIG = {}
if os.path.isfile(SERVER_CONFIG_FILE):
    import yaml
    SERVER_CONFIG = yaml.load(open(SERVER_CONFIG_FILE).read())

DEBUG = SERVER_CONFIG.get("debug", True)
TEMPLATE_DEBUG = SERVER_CONFIG.get("debug", True)

SERVER_HOME = SERVER_CONFIG.get("server_home", "/var/deploy")
SERVER_ROOT = SERVER_CONFIG.get("server_root", '')
LOGIN_URL="%s/accounts/login" % SERVER_ROOT
LOGIN_REDIRECT_URL="%s/accounts/profile" % SERVER_ROOT

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.sites',
    'bombardier_server.web.rest_api.configs',
)
SITE_ID=1
ROOT_URLCONF = 'bombardier_server.web.rest_api.urls'

DATABASE_NAME = os.path.join(SERVER_HOME, 'admin', 'cnmdb')
DATABASE_ENGINE = 'sqlite3'
TEMPLATE_DIRS = ["/usr/local/share/bdr-templates"]
TEST_DATABASE_NAME = 'testdb'
DATABASE_SUPPORTS_TRANSACTIONS = False
DATABASE_OPTIONS = {'timeout': 30}

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
#    'django.middleware.common.CommonMiddleware',
#    'django.middleware.doc.XViewMiddleware',
)
